#!/bin/sh
# ----------------------------------------------------------------------------
# /tmp/install.sh - nginx installation
#
# Creation   : 2014-05-11 ap
# Last update: $Id: install.sh 83702 2021-02-17 12:46:14Z ansgar $
#
# Copyright (c) 2001-2021 Ansgar Puester, ansgar.puester(at)freenet(dot)de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib

#exec 2>/tmp/install.sh-$$.log
#set -x

# set package name
packageName=nginx
packageNameLong='nginx web server'
package_pos=50

# ----------------------------------------------------------------------------
# Setup step 1 / create start stop links
# ----------------------------------------------------------------------------

### install startup/stop-scripts
package_start='S'${package_pos}${packageName}
package_kill='K'`/usr/bin/expr 100 - $package_pos`${packageName}

### symbolic links
mecho --std "creating symbolic links for init scripts ..."

rm -f /etc/rc2.d/[KS]??$packageName

ln -sf /etc/init.d/$packageName /etc/rc2.d/$package_kill
ln -sf /etc/init.d/$packageName /etc/rc2.d/$package_start

# ----------------------------------------------------------------------------
# Setup step 2 User and Group add
# ----------------------------------------------------------------------------

user='nginx'
group='nginx'
name='Nginx deamon'
homedir=''
shell='/bin/false'

# check group
if ! getent group ${group} >/dev/null 2>&1
then
    # mecho "adding group '${group}' ..."
    /var/install/bin/add-group ${group}
fi

# check user
if ! getent passwd ${user} >/dev/null 2>&1
then
    # mecho "adding user '${user}' ..."
    /var/install/bin/add-user -d ${user} "*" '' "${group}" "${name}" "${homedir}" "${shell}"
fi

# check password to make sure that the account has not been disabled
#invalid=$(getent shadow ${user} | grep "^${user}:!")
#if [ -n "$invalid" ]
#then
#   passwd -d ${user} >/dev/null 2>&1
#fi

# check password to make sure that the password was not deleted
deleted=$(getent shadow ${user} | grep "^${user}::")
if [ -n "$deleted" ]
then
   passwd -l ${user} >/dev/null 2>&1
fi


# ----------------------------------------------------------------------------
# Setup step 3
# ----------------------------------------------------------------------------	

# create directories
[ -d /var/lib/nginx ]         || mkdir /var/lib/nginx
[ -d /var/lib/nginx/tmp ]     || mkdir /var/lib/nginx/tmp
[ -d /var/lib/nginx/fastcgi ] || mkdir /var/lib/nginx/fastcgi
[ -d /var/lib/nginx/proxy ]   || mkdir /var/lib/nginx/proxy
[ -d /var/lib/nginx/scgi ]    || mkdir /var/lib/nginx/scgi
[ -d /var/lib/nginx/uwsgi ]   || mkdir /var/lib/nginx/uwsgi
[ -d /var/log/nginx ]         || mkdir /var/log/nginx

chmod 700 /var/lib/nginx/tmp
chmod 700 /var/lib/nginx/fastcgi
chmod 700 /var/lib/nginx/proxy
chmod 700 /var/lib/nginx/scgi
chmod 700 /var/lib/nginx/uwsgi
chmod 755 /var/log/nginx

chown nginx:nginx /var/lib/nginx/tmp
chown nginx:nginx /var/lib/nginx/fastcgi
chown nginx:nginx /var/lib/nginx/proxy
chown nginx:nginx /var/lib/nginx/scgi
chown nginx:nginx /var/lib/nginx/uwsgi

chown nginx:nginx /var/log/nginx

#chown nobody:root /var/lib/nginx/tmp
#chown nobody:root /var/lib/nginx/fastcgi
#chown nobody:root /var/lib/nginx/proxy
#chown nobody:root /var/lib/nginx/scgi
#chown nobody:root /var/lib/nginx/uwsgi
#chown root:root   /var/log/nginx

# ----------------------------------------------------------------------------
# Add menu
# ----------------------------------------------------------------------------

/var/install/bin/add-menu setup.services.menu setup.services.${packageName}.menu "${packageNameLong}"

# ----------------------------------------------------------------------------
# Create default config and other
# ----------------------------------------------------------------------------

/var/install/config.d/${packageName}-update.sh

# ----------------------------------------------------------------------------
# Stop, configure and start nginx
# ----------------------------------------------------------------------------
. /etc/config.d/${packageName}
if [ "${START_NGINX}" == 'yes' ]
then
    mecho --info "Stopping service"
    sh /etc/init.d/nginx stop

    /var/install/config.d/${packageName}.sh --quiet

    mecho --info "Starting service"
    sh /etc/init.d/nginx start
fi

exit 0
# ----------------------------------------------------------------------------
# End
# ----------------------------------------------------------------------------
