#!/bin/sh
# ----------------------------------------------------------------------------
# /var/install/config.d/nginx-update.sh - paramater update script
#
# Copyright (c) 2001-2021 Ansgar Puester <ansgar.puester@freenet.de
#
# Creation:     01.05.2015  ap
#
# Last update: $Id: nginx-update.sh 84681 2021-05-03 15:18:46Z ansgar $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

#exec 2> `pwd`/nginx-update-trace$$.log
#set -x

# include configlib for using printvar
. /var/install/include/configlib

# include eislib
. /var/install/include/eislib

# include config_shlib
. /var/install/bin/config_shlib

# set packages name
packageName=nginx

VERSION='0.2.6'

# ----------------------------------------------------------------------------
# Set the default values for configuration
# ----------------------------------------------------------------------------

START_NGINX='no'
NGINX_MAN_CONFIG='no'
NGINX_SERVER_TYPE='reverse_proxy'
NGINX_WORKER_PROCESSES='auto'
NGINX_WORKER_CONNECTIONS='1024'
NGINX_SERVER_LISTEN_N=1
NGINX_SERVER_LISTEN_1='80'
NGINX_SERVER_NAME='localhost'
NGINX_ERROR_LOG='error.log'
NGINX_ACCESS_LOG='access.log'
NGINX_LOGANON='no'
NGINX_LOGANON_MODE='16Bit'
NGINX_ROTATE_LOGS='yes'
NGINX_LOGROTATE_COUNT='6'
NGINX_LOGROTATE_INTERVAL='daily'
NGINX_STUB_STATUS_LOAD='no'
NGINX_STUB_STATUS_LOCATION='/basic_status'
NGINX_PROXY_BUFFERING='off'
NGINX_PROXY_PASS_URL='http://mydestination:80'
NGINX_UPSTREAM_NAME='myupstream'
NGINX_LB_SERVER_N='0'
NGINX_LB_SERVER_N_DEFAULT='2'
NGINX_LB_SERVER_1_ADDRESS='backend1.example.com'
NGINX_LB_SERVER_1_WEIGHT='1'
NGINX_LB_SERVER_1_MAX_FAILS='1'
NGINX_LB_SERVER_1_FAIL_TIMEOUT='10'
NGINX_LB_SERVER_1_BACKUP='no'
NGINX_LB_SERVER_2_ADDRESS='backend2.example.com'
NGINX_LB_SERVER_2_WEIGHT='1'
NGINX_LB_SERVER_2_MAX_FAILS='1'
NGINX_LB_SERVER_2_FAIL_TIMEOUT='10'
NGINX_LB_SERVER_2_BACKUP='no'

# ----------------------------------------------------------------------------
# Read old configuration
# ----------------------------------------------------------------------------
readOldVariables()
{
    # read old values
    if [ -f /etc/config.d/${packageName} ]
    then
        backup_config ${packageName} /etc/config.d/${packageName}
        . /etc/config.d/${packageName}

        # strip /logs/ from NGINX_ERROR_LOG and NGINX_ACCESS_LOG
        NGINX_ERROR_LOG=$(echo $NGINX_ERROR_LOG | sed 's|^/logs/||')
        NGINX_ACCESS_LOG=$(echo $NGINX_ACCESS_LOG | sed 's|^/logs/||')

    fi
}

# ----------------------------------------------------------------------------
# Write config and default files
# ----------------------------------------------------------------------------
makeConfigFile()
{
    internal_conf_file=${1}
    internal_conf_file_type=${2}
    {
    # ------------------------------------------------------------------------
    printgpl -conf ${packageName} "2015-05-01" "Ansgar Puester"
    # ------------------------------------------------------------------------

    echo "# Package ${packageName} Version ${VERSION}"

    #------------------------------------------------------------------------------
    printgroup "nginx web server (general settings)"
    #------------------------------------------------------------------------------

    printvar "START_NGINX"
    printvar ''                   "Start nginx 'yes' or 'no'."

    #------------------------------------------------------------------------------
    printgroup "nginx web server (configuration action settings)"
    #------------------------------------------------------------------------------

    printvar "NGINX_MAN_CONFIG"
    printvar ''                   "Configure nginx manually 'yes' or 'no'."
    printvar ''                   "If set to 'yes' configuration file /etc/nginx/nginx.conf will not be generated."
    printvar ''                   "It is your responsibility to manually create a valid configuration file."

    #------------------------------------------------------------------------------
    printgroup "nginx web server (general configuration settings)"
    #------------------------------------------------------------------------------

    printvar "NGINX_SERVER_TYPE"
    printvar ''                   "Server type:"
    printvar ''                   "reverse_proxy nginx will work as a reverse proxy"
    printvar ''                   "load_balancer nginx will work as a load balancer"
    printvar ''                   "Configuration for server type web_server will probably be avaliable in the future."
    printvar ''                   "Use NGINX_MAN_CONFIG='yes' to configure a web server."

    #------------------------------------------------------------------------------
    printgroup "nginx web server (configuration settings)"
    #------------------------------------------------------------------------------

    printvar "NGINX_WORKER_PROCESSES"
    printvar ''                       "Defines the number of worker processes."
    printvar ''                       "The optimal value depends on many factors including (but not limited to) the number of CPU cores, the"
    printvar ''                       "number of hard disk drives that store data, and load pattern. When one is in doubt, setting it to the"
    printvar ''                       "number of available CPU cores would be a good start (the value 'auto' will try to autodetect it)."
    printvar "NGINX_WORKER_CONNECTIONS"
    printvar ''                         "Sets the maximum number of simultaneous connections that can be opened by a worker process."
    printvar ''                         "It should be kept in mind that this number includes all connections (e.g. connections with proxied"
    printvar ''                         "servers, among others), not only connections with clients. Another consideration is that the actual"
    printvar ''                         "number of simultaneous connections cannot exceed the current limit on the maximum number of open files,"
    printvar ''                         "which can be changed by worker_rlimit_nofile."
    printvar "NGINX_SERVER_LISTEN_N"    "Number of listen directives"
idx=1
while [ "${idx}" -le "${NGINX_SERVER_LISTEN_N}" ]
do
    printvar "NGINX_SERVER_LISTEN_${idx}"
    if [ "${idx}" = 1 ]
    then
       printvar ''                    "Sets the address and port for IP, or the path for a UNIX-domain socket on which the server will accept"
       printvar ''                    "requests. Both address and port, or only VSDE address or only port can be specified. An address may also"
       printvar ''                    "be a hostname, for example:"
       printvar ''                    "  127.0.0.1:8000"
       printvar ''                    "  127.0.0.1"
       printvar ''                    "  8000"
       printvar ''                    "  *:8000"
       printvar ''                    "  localhost:8000"
       printvar ''                    "IPv6 addresses (0.7.36) are specified in square brackets:"
       printvar ''                    "  [::]:8000"
       printvar ''                    "  [::1]"
       printvar ''                    "UNIX-domain socket:"
       printvar ''                    "  unix:/var/run/nginx_sock"
    fi
let idx=$idx+1
done
    printvar "NGINX_SERVER_NAME"
    printvar ''                    "Sets names of a virtual server."

    #------------------------------------------------------------------------------
    printgroup "nginx web server (log configuration settings)"
    #------------------------------------------------------------------------------

    printvar "NGINX_ERROR_LOG"
    printvar ''                   "Logfile name and level. A relative filename will be interpreted relativ to /var/log/nginx/."
    printvar ''                   "Configure a syslog server with syslog:server=address."
    printvar ''                   "It's NOT possible to disable error logging using NGINX_ERROR_LOG='off'."
    printvar "NGINX_ACCESS_LOG"
    printvar ''                   "Logfile name and level. A relative filename will be interpreted relativ to /var/log/nginx/."
    printvar ''                   "'off' disables access loging. Configure a syslog server with syslog:server=address."
    printvar "NGINX_LOGANON"
    printvar ''                   "Anonymize IP logging in nginx acccess log 'yes' or 'no'"
    printvar "NGINX_LOGANON_MODE"
    printvar ''                   "Keep 16Bit or 24Bit"
    printvar "NGINX_ROTATE_LOGS"
    printvar ''                   "Use logrotate to rotate the nginx error log and access log."
    printvar ''                   "Check if NGINX_ROTATE_LOGS has to be 'no' when using NGINX_MAN_CONFIG='yes'."
    printvar "NGINX_LOGROTATE_COUNT"
    printvar ''                   "Number of logs to save."
    printvar "NGINX_LOGROTATE_INTERVAL"
    printvar ''                   "Interval: daily, weekly, monthly"

    #------------------------------------------------------------------------------
    printgroup "nginx web server (status information settings)"
    #------------------------------------------------------------------------------

    printvar "NGINX_STUB_STATUS_LOAD"
    printvar ''                   "# Load stub status module 'yes' or 'no'."
                                  
    printvar "NGINX_STUB_STATUS_LOCATION"
    printvar ''                   "# Location of stub status."
    printvar ''                   "# You can acces the status via http://<servername or serverip><NGINX_STUB_STATUS_LOCATION>"
    printvar ''                   "# E.g. http://eisgcc.ap.de/basic_status"

    #------------------------------------------------------------------------------
    printgroup "nginx reverse proxy (configuration settings)"
    #------------------------------------------------------------------------------

    printvar "NGINX_PROXY_BUFFERING"
    printvar ''                   "Enables or disables buffering of responses from the proxied server. Values 'on' or 'off'."
    printvar "NGINX_PROXY_PASS_URL"
    printvar ''                   "Sets the protocol and address of a proxied server."

    #------------------------------------------------------------------------------
    printgroup "nginx load balancer (configuration settings)"
    #------------------------------------------------------------------------------

    printvar "NGINX_UPSTREAM_NAME"
    printvar ''                   "Name of upstream definition."
    printvar "NGINX_LB_SERVER_N"
    printvar ''                   "Number of server in upstream definition."
if [ "$internal_conf_file_type" = 'DEFAULT' ]
then
   nginx_lb_server_count=$NGINX_LB_SERVER_N_DEFAULT
else
   nginx_lb_server_count=$NGINX_LB_SERVER_N
fi
idx=1
while [ "${idx}" -le "${nginx_lb_server_count}" ]
do
    printvar "NGINX_LB_SERVER_${idx}_ADDRESS"
    [ "${idx}" = 1 ] && printvar ''                                    "Address of nth server."
    printvar "NGINX_LB_SERVER_${idx}_WEIGHT"
    [ "${idx}" = 1 ] && printvar ''                                    "Weight of nth server."
    printvar "NGINX_LB_SERVER_${idx}_MAX_FAILS"
    [ "${idx}" = 1 ] && printvar ''                                    "Maximum fails of nth server."
    printvar "NGINX_LB_SERVER_${idx}_FAIL_TIMEOUT"
    [ "${idx}" = 1 ] && printvar ''                                    "Fail timeout of nth server in seconds."
    printvar "NGINX_LB_SERVER_${idx}_BACKUP"
    [ "${idx}" = 1 ] && printvar ''                                    "Is nth server a backup server 'yes' or 'no'."
let idx=$idx+1
done

    #------------------------------------------------------------------------------
    printend
    #------------------------------------------------------------------------------

    } > ${internal_conf_file}
    # Set rights
    chmod 0600 ${internal_conf_file}
    chown root ${internal_conf_file}
}

# ----------------------------------------------------------------------------
# Create the check.d file
# ----------------------------------------------------------------------------
makeCheckFile()
{
    printgpl -check ${packageName} '2015-05-01' 'Ansgar Puester' >/etc/check.d/${packageName}
    cat >> /etc/check.d/${packageName} <<EOFG
# Variable                     OPT_VARIABLE                   VARIABLE_N                     VALUE
START_NGINX                    -                              -                              YESNO
NGINX_MAN_CONFIG               START_NGINX                    -                              YESNO
NGINX_SERVER_TYPE              !NGINX_MAN_CONFIG              -                              NGINX_SERVER_TYPE_DEF
NGINX_WORKER_PROCESSES         !NGINX_MAN_CONFIG              -                              NGINX_WORKER_PROCESSES_DEF
NGINX_WORKER_CONNECTIONS       !NGINX_MAN_CONFIG              -                              NGINX_NUMERIC_NOT_0_DEF
NGINX_SERVER_LISTEN_N          !NGINX_MAN_CONFIG              -                              NUMERIC
NGINX_SERVER_LISTEN_%          !NGINX_MAN_CONFIG              NGINX_SERVER_LISTEN_N          NGINX_SERVER_LISTEN_DEF
NGINX_SERVER_NAME              !NGINX_MAN_CONFIG              -                              NOBLANK
NGINX_ERROR_LOG                START_NGINX                    -                              NOBLANK
NGINX_ACCESS_LOG               START_NGINX                    -                              NOBLANK
NGINX_LOGANON                  START_NGINX                    -                              YESNO
NGINX_LOGANON_MODE             NGINX_LOGANON                  -                              NGINX_LOGANON_MODE_TYPE_DEF
NGINX_ROTATE_LOGS              START_NGINX                    -                              YESNO
NGINX_LOGROTATE_COUNT          START_NGINX                    -                              NOBLANK
NGINX_LOGROTATE_INTERVAL       START_NGINX                    -                              NOBLANK
NGINX_STUB_STATUS_LOAD         !NGINX_MAN_CONFIG              -                              YESNO
NGINX_STUB_STATUS_LOCATION     !NGINX_MAN_CONFIG              -                              NOBLANK
NGINX_PROXY_BUFFERING          !NGINX_MAN_CONFIG              -                              NGINX_PROXY_BUFFERING_DEF
NGINX_PROXY_PASS_URL           !NGINX_MAN_CONFIG              -                              NONE
NGINX_UPSTREAM_NAME            !NGINX_MAN_CONFIG              -                              NOBLANK
NGINX_LB_SERVER_N              !NGINX_MAN_CONFIG              -                              NUMERIC
NGINX_LB_SERVER_%_ADDRESS      !NGINX_MAN_CONFIG              NGINX_LB_SERVER_N              NOBLANK
NGINX_LB_SERVER_%_WEIGHT       !NGINX_MAN_CONFIG              NGINX_LB_SERVER_N              NGINX_ENUMERIC_NOT_0_DEF
NGINX_LB_SERVER_%_MAX_FAILS    !NGINX_MAN_CONFIG              NGINX_LB_SERVER_N              NGINX_ENUMERIC_NOT_0_DEF
NGINX_LB_SERVER_%_FAIL_TIMEOUT !NGINX_MAN_CONFIG              NGINX_LB_SERVER_N              NGINX_ENUMERIC_NOT_0_DEF
NGINX_LB_SERVER_%_BACKUP       !NGINX_MAN_CONFIG              NGINX_LB_SERVER_N              YESNO
EOFG

    # Set rights for check.d file
    chmod 0600 /etc/check.d/${packageName}
    chown root /etc/check.d/${packageName}

    printgpl -check_exp ${packageName} '2015-05-01' 'Ansgar Puester' >/etc/check.d/${packageName}.exp
    cat >> /etc/check.d/${packageName}.exp <<EOFG

NGINX_SERVER_TYPE_DEF         = 'reverse_proxy|load_balancer'
                              : 'only reverse_proxy or load_balancere are allowed'
NGINX_NUMERIC_NOT_0_DEF       = '[1-9]|[1-9][0-9]+'
                              : 'value 0 not allowed'
NGINX_ENUMERIC_NOT_0_DEF      = '()|(RE:NGINX_NUMERIC_NOT_0_DEF)'
                              : 'value 0 not allowed'
NGINX_WORKER_PROCESSES_DEF    = 'auto|(RE:NGINX_NUMERIC_NOT_0_DEF)'
                              : 'only auto or numeric value allowed'
NGINX_LOGANON_MODE_TYPE_DEF   = '16Bit|24Bit'
                              : 'only 16Bit or 24Bit is allowed'
NGINX_PROXY_BUFFERING_DEF     = 'on|off'
                              : 'only on or off is allowed'
NGINX_SERVER_LISTEN_IPV4_DEF  = '(RE:PORT)|(RE:IPADDR)|(RE:IPADDR):(RE:PORT)'
                              : 'invalid specification, see documentation'
NGINX_SERVER_LISTEN_FQDN_DEF  = '(RE:FQDN)|(RE:FQDN):(RE:PORT)'
                              : 'invalid specification, see documentation'
NGINX_SERVER_LISTEN_HN_DEF    = '(RE:HOSTNAME)|(RE:HOSTNAME):(RE:PORT)'
                              : 'invalid specification, see documentation'
NGINX_SERVER_LISTEN_SOCK_DEF  = 'unix:(RE:ABS_PATH)'
                              : 'invalid specification, see documentation'
NGINX_SERVER_LISTEN_IPV6_DEF  = '\\[(RE:IPADDR6)\\]|\\[(RE:IPADDR6)\\]:(RE:PORT)'
                              : 'invalid specification, see documentation'
NGINX_SERVER_LISTEN_DEF       = '(RE:NGINX_SERVER_LISTEN_IPV4_DEF)|(RE:NGINX_SERVER_LISTEN_IPV6_DEF)|(RE:NGINX_SERVER_LISTEN_FQDN_DEF)|(RE:NGINX_SERVER_LISTEN_HN_DEF)|(RE:NGINX_SERVER_LISTEN_SOCK_DEF)'
                              : 'invalid specification, see documentation'
EOFG

    # Set rights for check.exp file
    chmod 0600 /etc/check.d/${packageName}.exp
    chown root /etc/check.d/${packageName}.exp

    printgpl -check_ext ${packageName} '2015-05-01' 'Ansgar Puester' >/etc/check.d/${packageName}.ext
    cat >> /etc/check.d/${packageName}.ext <<EOFG

set b_counter = 0
if ( start_nginx != "yes" && start_nginx != "no" )
then
   error "START_NGINX must be 'yes' or 'no'"
fi
if ( nginx_server_type == "load_balancer" && nginx_lb_server_n == "0" )
then
   error "NGINX_SERVER_TYPE was set to 'load_balancer' but NGINX_LB_SERVER_N is '0'"
   foreach i in nginx_lb_server_n
   do
      if ( nginx_lb_server_%_backup[i] == "yes" )
      then
         set b_counter = b_counter + 1
      fi
   done
   if ( b_counter == nginx_lb_server_n )
   then 
      error "All load balancer server are defined as 'backup'. That's impossible."
   fi
fi

EOFG

    # Set rights for check.ext file
    chmod 0600 /etc/check.d/${packageName}.ext
    chown root /etc/check.d/${packageName}.ext
}

# ----------------------------------------------------------------------------
# Main
# ----------------------------------------------------------------------------

# Write default config file
if [ -f /etc/config.d/${packageName} ]
then
    mecho --info "Updating main configuration."
else
    mecho --info "Creating main configuration."
fi

# check for old configuration
# check if NGINX_SERVER_LISTEN_N is defined
if [[ ! -v NGINX_SERVER_LISTEN_N ]]
then
   NGINX_SERVER_LISTEN_N='1'
   NGINX_SERVER_LISTEN_1="$NGINX_SERVER_LISTEN"
fi

makeConfigFile /etc/default.d/${packageName} DEFAULT

# Update from old version
readOldVariables

# Write new config file
makeConfigFile /etc/config.d/${packageName}

# Write check.d file
makeCheckFile

mecho --info "Finished."

exit 0
# ----------------------------------------------------------------------------
# End
# ----------------------------------------------------------------------------
